import { fromJS } from 'immutable'
import diff from 'deep-diff'
import { EventEmitter2 } from 'eventemitter2'

const changeMap = {
  N: 'NEW',
  D: 'DELETED',
  E: 'EDITED',
  A: 'ALTERED'
}

function logGroupStart (group) {
  if (!console.group) return console.log('--- change ---')
  console.group('change')
}

function logGroupEnd (group) {
  if (!console.group) return console.log('--- change end ---')
  console.groupEnd('change end')
}

function stringify (item) {
  return typeof item === 'object' ? JSON.stringify(item) : item
}

const describeChange = origin => {
  const lhs = stringify(origin.lhs)
  const rhs = stringify(origin.rhs)
  const changeDesc = changeMap[origin.kind]

  if (origin.kind === 'A') {
    console.log(`${changeMap[origin.kind]} at index: ${origin.index}`)
    return describeChange(origin.item)
  } else if (origin.path) {
    const path = `${origin.path.join('.')}`

    return `${changeDesc} '${path}': ${lhs} -> ${rhs}`
  }

  return `${changeDesc}: ${lhs} -> ${rhs}`
}

export const describeDiff = difference => {
  logGroupStart('change')
  difference.forEach(i => console.log(describeChange(i)))
  logGroupEnd('change')
}

function getDiff (a, b) {
  const difference = diff(a, b)

  if (!difference) {
    return []
  }

  difference.sort(i => {
    return i.kind < i.kind
  })

  return difference
}

export default function monitorState (getter, delay) {
  const monitor = new EventEmitter2()
  let snapshot = fromJS(getter())
  let timeout

  function check () {
    const newValue = getter()
    const difference = getDiff(snapshot.toJS(), newValue)

    if (difference.length) {
      monitor.emit('change', difference)
    }

    snapshot = fromJS(newValue)
    timeout = setTimeout(check, delay)
  }

  check()
  monitor.on('stop', () => clearTimeout(timeout))

  return monitor
}
